@@ -25,15 +25,14 @@
  * SUCH DAMAGE.
  */
 
+#include "config/config.h"
+
 #include <sys/param.h>
-#include <sys/mac.h>
 #include <sys/queue.h>
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <sys/mount.h>
-
-#include <rpcsvc/mount.h>
-#include <rpcsvc/nfs_prot.h>
+#include <sys/statvfs.h>
 
 #include <ctype.h>
 #include <errno.h>
@@ -43,7 +42,15 @@
 #include <string.h>
 #include <unistd.h>
 
-#include "nfs_label.h"
+#if !HAVE_DIGITTOINT
+#include "compat/digittoint.h"
+#endif
+#if !HAVE_STRLCPY
+#include "compat/strlcpy.h"
+#endif
+
+#include "mount.h"
+#include "nfs_prot.h"
 #include "libnfs.h"
 
 /*
@@ -309,12 +316,6 @@ nfs_mount_free(struct nfsmount *nfsm)
 		auth_destroy(nfsm->nm_mntauth);
 		nfsm->nm_mntauth = NULL;
 	}
-#ifdef LABELS
-	if (nfsm->nm_lblclient != NULL) {
-		nfsm->nm_lblclient->cl_auth = NULL;
-		clnt_destroy(nfsm->nm_lblclient);
-	}
-#endif
 	if (nfsm->nm_nfsclient != NULL) {
 		nfsm->nm_nfsclient->cl_auth = NULL;
 		clnt_destroy(nfsm->nm_nfsclient);
@@ -415,12 +416,18 @@ nfs_fattr3_to_stat(fattr3 *attr, struct stat *sbp)
 	sbp->st_uid = attr->uid;
 	sbp->st_gid = attr->gid;
 	/* XXXRW: sbp->st_rdev =  attr->rdev; */
+#if HAVE_STRUCT_STAT_ST_ATIMESPEC
 	sbp->st_atimespec.tv_sec = attr->atime.seconds;
 	sbp->st_atimespec.tv_nsec = attr->atime.nseconds;
 	sbp->st_mtimespec.tv_sec = attr->mtime.seconds;
 	sbp->st_mtimespec.tv_nsec = attr->mtime.nseconds;
 	sbp->st_ctimespec.tv_sec = attr->ctime.seconds;
 	sbp->st_ctimespec.tv_nsec = attr->ctime.nseconds;
+#else
+	sbp->st_atime = attr->atime.seconds;
+	sbp->st_mtime = attr->mtime.seconds;
+	sbp->st_ctime = attr->ctime.seconds;
+#endif
 	sbp->st_size = attr->size;
 	sbp->st_blocks = attr->used / NFS_BLOCKSIZE;
 	if (attr->used % NFS_BLOCKSIZE != 0)
@@ -490,9 +497,10 @@ nfs_hex2fh(const char *hexstring, nfs_fh3 *fhp)
  * NFS authenticators.  If an explicit file handle is passed, we ignore the
  * requested path, and don't perform the mount (or later unmount) RPC.
  */
-static int
-nfs_mount_internal(struct nfsmount **nfsmp, struct nfscred *cred,
-    const char *hostname, const char *path, const char *fhandle)
+int
+nfs_mount(struct nfsmount **nfsmp, struct nfscred *cred,
+    const char *hostname, int mounttype, const char *src,
+    const char *nettype)
 {
 	char local_hostname[_POSIX_HOST_NAME_MAX];
 	struct nfsmount *nfsm;
@@ -500,16 +508,13 @@ nfs_mount_internal(struct nfsmount **nfsmp, struct nfscred *cred,
 	nfs_fh3 tmp_fh;
 	int error;
 
-	if (fhandle != NULL)
-		path = "";
-
 	if (gethostname(local_hostname, _POSIX_HOST_NAME_MAX) == -1)
 		return (-1);
 
-	if (nfs_mount_alloc(&nfsm, hostname, path) == -1)
+	if (nfs_mount_alloc(&nfsm, hostname, src) == -1)
 		return (-1);
 
-	nfsm->nm_nfsauth = authsys_create(local_hostname, cred->nc_uid,
+	nfsm->nm_nfsauth = authunix_create(local_hostname, cred->nc_uid,
 	    cred->nc_gid, cred->nc_len, cred->nc_groups);
 	if (nfsm->nm_nfsauth == NULL) {
 		/* XXXRW: How to handle error? */
@@ -517,13 +522,15 @@ nfs_mount_internal(struct nfsmount **nfsmp, struct nfscred *cred,
 		return (-1);
 	}
 
-	if (fhandle == NULL) {
+	switch (mounttype) {
+	case NFSMOUNT_TYPE_PATH:
 		/*
 		 * Build an RPC connection to register a mount and query the
 		 * root file handle.
 		 */
-		nfsm->nm_mntauth = authsys_create(local_hostname, cred->nc_uid,
-		    cred->nc_gid, cred->nc_len, cred->nc_groups);
+		nfsm->nm_mntauth = authunix_create(local_hostname,
+		    cred->nc_uid, cred->nc_gid, cred->nc_len,
+		    cred->nc_groups);
 		if (nfsm->nm_mntauth == NULL) {
 			/* XXXRW: How to handle error? */
 			nfs_mount_free(nfsm);
@@ -531,7 +538,7 @@ nfs_mount_internal(struct nfsmount **nfsmp, struct nfscred *cred,
 		}
 
 		nfsm->nm_mntclient = clnt_create(nfsm->nm_hostname,
-		    MOUNTPROG, MOUNTVERS3, "udp");
+		    MOUNTPROG, MOUNTVERS3, nettype);
 		if (nfsm->nm_mntclient == NULL) {
 			clnt_pcreateerror("mntclient");
 			/* XXXRW: How to handle error? */
@@ -565,14 +572,15 @@ nfs_mount_internal(struct nfsmount **nfsmp, struct nfscred *cred,
 			errno = error;
 			return (-1);
 		}
-	} else {
+		break;
 
+	case NFSMOUNT_TYPE_HANDLE:
 		/*
 		 * File handle passed explicitly as an nfs_mount_fh()
 		 * argument, convert from hex and install.  We'll validate
 		 * with a later call to fsinfo.
 		 */
-		if (nfs_hex2fh(fhandle, &tmp_fh) == -1) {
+		if (nfs_hex2fh(src, &tmp_fh) == -1) {
 			error = errno;
 			nfs_mount_free(nfsm);
 			errno = error;
@@ -586,10 +594,16 @@ nfs_mount_internal(struct nfsmount **nfsmp, struct nfscred *cred,
 			return (-1);
 		}
 		free(tmp_fh.data.data_val);
+		break;
+
+	default:
+		nfs_mount_free(nfsm);
+		errno = EINVAL;
+		return (-1);
 	}
 
 	nfsm->nm_nfsclient = clnt_create(nfsm->nm_hostname, NFS3_PROGRAM,
-	    NFS_V3, "udp");
+	    NFS_V3, nettype);
 	if (nfsm->nm_nfsclient == NULL) {
 		clnt_pcreateerror("nfsclient");
 		/* XXXRW: How to handle error? */
@@ -613,44 +627,10 @@ nfs_mount_internal(struct nfsmount **nfsmp, struct nfscred *cred,
 		return (-1);
 	}
 
-#ifdef LABELS
-	/*
-	 * Set up the label query service.  At this point in development, we
-	 * allow successful access to labels to be considered optional, so
-	 * keep going if the call fails.
-	 */
-	nfsm->nm_lblclient = clnt_create(nfsm->nm_hostname,
-	    NFS3_LABEL_PROGRAM, NFS_LABEL_V3, "udp");
-	if (nfsm->nm_lblclient == NULL)
-		clnt_pcreateerror("lblclient");
-
-	/*
-	 * Perform a NULL label operation to test to see if the daemon is
-	 * alive.  Warn if we get an error.
-	 */
-	if (nfsm->nm_lblclient != NULL && nfslabel_null(nfsm) != 0)
-		perror("nfslabel_null");
-#endif
-
 	*nfsmp = nfsm;
 	return (0);
 }
 
-int
-nfs_mount(struct nfsmount **nfsmp, struct nfscred *cred,
-    const char *hostname, const char *path)
-{
-
-	return (nfs_mount_internal(nfsmp, cred, hostname, path, NULL));
-}
-
-int
-nfs_mount_fh(struct nfsmount **nfsmp, struct nfscred *cred,
-    const char *hostname, const char *fhandle)
-{
-
-	return (nfs_mount_internal(nfsmp, cred, hostname, NULL, fhandle));
-}
 /*
  * Modify the credential associated with an RPC session.
  */
@@ -663,7 +643,7 @@ nfs_setcred(struct nfsmount *nfsm, struct nfscred *cred)
 	if (gethostname(local_hostname, _POSIX_HOST_NAME_MAX) == -1)
 		return (-1);
 
-	auth = authsys_create(local_hostname, cred->nc_uid, cred->nc_gid,
+	auth = authunix_create(local_hostname, cred->nc_uid, cred->nc_gid,
 	    cred->nc_len, cred->nc_groups);
 	if (auth == NULL) {
 		/* XXXRW: Return an errno? */
@@ -1741,11 +1721,11 @@ nfs_fsync(struct nfsmount *nfsm, struct nfsnode *node)
 }
 
 /*
- * Implement statfs(2)-like behavior using the NFSv3 NFSPROC3_FSSTAT RPC.
+ * Implement statvfs(3)-like behavior using the NFSv3 NFSPROC3_FSSTAT RPC.
  * Returns (-1) on failure, (0) on success.
  */
 int
-nfs_statfs(struct nfsmount *nfsm, struct statfs *buf)
+nfs_statvfs(struct nfsmount *nfsm, struct statvfs *buf)
 {
 	FSSTAT3res *result;
 	FSSTAT3args args;
@@ -1764,29 +1744,20 @@ nfs_statfs(struct nfsmount *nfsm, struct statfs *buf)
 	}
 
 	bzero(buf, sizeof(*buf));
-	buf->f_version = STATFS_VERSION;
-	buf->f_type = 0;
-	buf->f_flags = 0;
-	buf->f_bsize = NFS_BLOCKSIZE;
-	buf->f_iosize = nfsm->nm_nfsinfo.ni_rtpref;
-	buf->f_blocks = result->FSSTAT3res_u.resok.tbytes / NFS_BLOCKSIZE;
-	buf->f_bfree = result->FSSTAT3res_u.resok.fbytes / NFS_BLOCKSIZE;
-	/*
-	 * XXXRW: If our credential is a root credential, might return the
-	 * wrong value for "avail".
-	 */
+
 	buf->f_bavail = result->FSSTAT3res_u.resok.abytes / NFS_BLOCKSIZE;
-	buf->f_files = result->FSSTAT3res_u.resok.tfiles;
+	buf->f_bfree = result->FSSTAT3res_u.resok.fbytes / NFS_BLOCKSIZE;
+	buf->f_blocks = result->FSSTAT3res_u.resok.tbytes / NFS_BLOCKSIZE;
+
+	buf->f_favail = result->FSSTAT3res_u.resok.afiles;
 	buf->f_ffree = result->FSSTAT3res_u.resok.ffiles;
-	buf->f_syncwrites = 0;
-	buf->f_asyncwrites = 0;
-	buf->f_syncreads = 0;
-	buf->f_asyncreads = 0;
+	buf->f_files = result->FSSTAT3res_u.resok.tfiles;
+
+	buf->f_bsize = NFS_BLOCKSIZE;
+	buf->f_flag = 0;
+	buf->f_frsize = NFS_BLOCKSIZE;
 	buf->f_namemax = NFS_MAXNAMLEN;
-	strcpy(buf->f_fstypename, "nfs");
-	snprintf(buf->f_mntfromname, MNAMELEN, "%s:%s", nfsm->nm_hostname,
-	    nfsm->nm_path);
-	strcpy(buf->f_mntonname, "/");
+
 	return (0);
 }
 
@@ -1797,118 +1768,3 @@ nfs_pathconf(struct nfsmount *nfsm)
 	errno = EOPNOTSUPP;
 	return (-1);
 }
-
-/*
- * Functions implementing the McAfee Research NFSv3 label RPC extensions.
- */
-#ifdef LABELS
-static int
-nfs_label_to_errno(enum nfs3_label_status status)
-{
-
-	switch (status) {
-	case NFS3_LABEL_OK:
-		return (0);
-	case NFS3_LABEL_PERM:
-		return (EPERM);
-	case NFS3_LABEL_IO:
-		return (EIO);
-	case NFS3_LABEL_NOMEM:
-		return (ENOMEM);
-	case NFS3_LABEL_ACCES:
-		return (EACCES);
-	case NFS3_LABEL_OPNOTSUPP:
-		return (EOPNOTSUPP);
-	case NFS3_LABEL_STALE:
-		return (ESTALE);
-	case NFS3_LABEL_NOSYS:
-		return (ENOSYS);
-	default:
-		return (-1);
-	}
-}
-
-int
-nfslabel_null(struct nfsmount *nfsm)
-{
-
-	if (nfsm->nm_lblclient == NULL) {
-		errno = EOPNOTSUPP;
-		return (-1);
-	}
-
-	if (nfsproc3_label_null_3(NULL, nfsm->nm_lblclient) == NULL) {
-		/* XXXRW: Map to a more useful error? */
-		errno = EINVAL;
-		return (-1);
-	}
-	return (0);
-}
-
-int
-nfslabel_getlabel(struct nfsmount *nfsm, struct nfsnode *node, mac_t label)
-{
-	GETLABEL3res *result;
-	GETLABEL3args args;
-	int len;
-
-	if (nfsm->nm_lblclient == NULL) {
-		errno = EOPNOTSUPP;
-		return (-1);
-	}
-
-	bzero(&args, sizeof(args));
-	args.object.data.data_val = node->n_handle.data.data_val;
-	args.object.data.data_len = node->n_handle.data.data_len;
-	args.elements = label->m_string;
-
-	result = nfsproc3_label_getlabel_3(&args, nfsm->nm_lblclient);
-	if (result == NULL) {
-		/* XXXRW: Map to a more useful error? */
-		errno = EINVAL;
-		return (-1);
-	}
-	if (result->status != NFS3_LABEL_OK) {
-		errno = nfs_label_to_errno(result->status);
-		return (-1);
-	}
-	len = strlen(result->GETLABEL3res_u.resok.label) + 1;
-	if (len < label->m_buflen) {
-		char *string;
-
-		/*
-		 * Arguably, we should actually reject this, but for now
-		 * accept.
-		 */
-		string = malloc(len);
-		if (string == NULL) {
-			errno = ENOMEM;
-			return (-1);
-		}
-		free(label->m_string);
-		label->m_string = string;
-		label->m_buflen = len;
-	}
-	strcpy(label->m_string, result->GETLABEL3res_u.resok.label);
-	return (0);
-}
-
-int
-nfslabel_setlabel(struct nfsmount *nfsm, struct nfsnode *node, mac_t label)
-{
-#if 0
-	SETLABEL3res *result;
-#endif
-	SETLABEL3args *args;
-
-	if (nfsm->nm_lblclient == NULL) {
-		errno = EOPNOTSUPP;
-		return (-1);
-	}
-
-	bzero(&args, sizeof(args));
-
-	errno = EOPNOTSUPP;
-	return (-1);
-}
-#endif
